<?php

namespace Drupal\mcapi\Plugin\Block;

use Drupal\mcapi\Storage\WalletStorage;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;

/**
 * Displays all the wallets of an entity.
 *
 * Entity being either the current user OR the entity being viewed. Shows the
 * wallet view mode 'mini'.
 *
 * @Block(
 *   id = "mcapi_wallet_mini",
 *   admin_label = @Translation("Mini view of entity's wallets"),
 *   category = @Translation("Community Accounting")
 * )
 */
class WalletView extends McapiBlockBase {

    public function __construct(array $configuration, $plugin_id, $plugin_definition, $entityTypeManager, $currentUser, $current_route_match, $request,  EntityDisplayRepositoryInterface $entity_display_repository) {
      parent::__construct($configuration, $plugin_id, $plugin_definition, $entityTypeManager, $currentUser, $current_route_match, $request);
      $this->entityDisplayRepository = $entity_display_repository;
    }

  /**
   * Injection.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('current_route_match'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('entity_display.repository')
    );
  }


  public function defaultConfiguration() {
    $conf = parent::defaultConfiguration();
    $conf['view_mode'] = 'summary';
    return $conf;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    parent::blockForm($form, $form_state);
    $form['view_mode'] = [
      '#title' => $this->t('View mode'),
      '#type' => 'radios',
      '#options' => $this->entityDisplayRepository->getViewModeOptions('mcapi_wallet'),
      '#default_value' => $this->configuration['view_mode']
    ];
    unset($form['view_mode']['#options']['default'], $form['view_mode']['#options']['token']);
    $form['curr_ids'] = [
      '#type' => 'hidden',
      '#value' => []
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    return $this->entityTypeManager
      ->getViewBuilder('mcapi_wallet')
      ->viewMultiple(
        WalletStorage::walletsOf($this->holderEntity, TRUE),
        $this->configuration['view_mode']
      );
  }

}
