<?php

namespace Drupal\mcapi\Plugin\Condition;

use Drupal\rules\Core\RulesConditionBase;
use Drupal\mcapi\Entity\Currency;

/**
 * Provides a 'Worth field is more than' condition.
 *
 * @Condition(
 *   id = "mcapi_worth_more_than",
 *   label = @Translation("Entity is worth more than"),
 *   category = @Translation("Community Accounting"),
 *   provider = "rules",
 *   context_definitions = {
 *     "transaction" = @ContextDefinition("entity:mcapi_transaction",
 *       label = @Translation("Transaction"),
 *     ),
 *     "currency_id" = @ContextDefinition(
 *       "string",
 *       label = @Translation("Currency ID"),
 *       description = @Translation("the currency ID"),
 *     ),
 *     "quantity" = @ContextDefinition(
 *       "integer",
 *       label = @Translation("Quantity"),
 *       description = @Translation("the value of the worth field"),
 *     )
 *   }
 * )
 */
class EntityWorthMoreThan extends RulesConditionBase {

  /**
   * {@inheritdoc}
   *
   * This shows at the top of the condition editing page.
   */
  public function summary() {
    $conf = $this->getConfiguration();

    $summary = $this->t(
      'Entity is worth at least %value',
      ['%value' => Currency::load($conf['currency_id'])->format($conf['quantity'])]
    );
    return (string)$summary;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'worth' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function doEvaluate($entity, $fieldname, $worth) {
    $entity = $this->getContextValue('entity');
    $fieldname = $this->getContextValue('fieldname');
    $worth = $this->getContextValue('worth');
    if (is_null($worth->value)) {
      return TRUE;
    }
    elseif ($entity->{$fieldname}->getValue($worth->curr_id) > $worth->value) {
      return TRUE;
    }
  }

  /**
   * {@inheritdoc}
   *
   * @todo
   */
  public function negate($negate = TRUE) {
    return parent::negate($negate);
  }
}
