<?php

namespace Drupal\mcapi\Plugin\Field\FieldFormatter;

use Drupal\mcapi\Mcapi;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceLabelFormatter;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'wallet_name' formatter.
 *
 * @FieldFormatter(
 *   id = "wallet_name",
 *   label = @Translation("Wallet name"),
 *   field_types = {
 *     "wallet_reference"
 *   }
 * )
 */
class WalletNameFormatter extends EntityReferenceLabelFormatter {

  /**
   * {@inheritdoc}
   *
   * Wallet names can always be seen. Usually this is in the context of viewing
   * a transaction which has its own more detailed access control.
   */
  protected function checkAccess(EntityInterface $entity) {
    return AccessResult::allowed();
  }

  /**
   * {@inheritdoc}
   *
   * Replace the wallet link with the wallet owner link.
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $wallet) {
      $holder = $wallet->getHolder();
      $max = Mcapi::maxWalletsOfBundle($holder->getEntityTypeId(), $holder->bundle());
      if ($max < 2) {
        $elements = [];
        $uri = $holder->urlInfo();
        $elements[$delta] = [
          '#type' => 'link',
          '#title' => $holder->label(),
          '#url' => $uri,
          '#options' => $uri->getOptions(),
        ];
      }
      else {
        $elements = parent::viewElements($items, $langcode);
      }
    }
    return $elements;
  }
}
